import UIKit
import PlaygroundSupport

extension UIView {
    func constrainCenter(in view: UIView) {
        self.centerXAnchor.constraint(equalTo: view.centerXAnchor).isActive = true
        self.centerYAnchor.constraint(equalTo: view.centerYAnchor).isActive = true
    }
    func constrainSize(_ size: CGSize) {
        self.widthAnchor.constraint(equalToConstant: size.width).isActive = true
        self.heightAnchor.constraint(equalToConstant: size.height).isActive = true
    }
    func constrainEdges(in view: UIView) {
        self.topAnchor.constraint(equalTo: view.topAnchor).isActive = true
        self.bottomAnchor.constraint(equalTo: view.bottomAnchor).isActive = true
        self.leftAnchor.constraint(equalTo: view.leftAnchor).isActive = true
        self.rightAnchor.constraint(equalTo: view.rightAnchor).isActive = true
    }
}

class MyViewController : UIViewController {
    override func loadView() {
        let view = UIView()
        self.view = view
        view.backgroundColor = .white
        
        let font = UIFont(name: "chalkduster", size: 30)!
//        let font = UIFont.systemFont(ofSize: 40)
        let message = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas vel maximus tellus. Pellentesque in metus aliquam, condimentum dui ac, malesuada lacus. Maecenas consequat, enim at."
        
        let label = createLabel(font: font, message: message)
        view.addSubview(label)
        
        let textView = createTextView(font: font, message: message)
        view.addSubview(textView)
        
        label.constrainCenter(in: view)
        textView.constrainEdges(in: label)
    }
    
    private func createLabel(font: UIFont, message: String) -> UILabel {
        let label = UILabel()
        label.font = font
        label.text = message
        label.textColor = .blue
        
        label.sizeToFit()
        label.translatesAutoresizingMaskIntoConstraints = false
        
        label.layer.borderWidth = 1
        label.layer.borderColor = UIColor.blue.cgColor

        label.preferredMaxLayoutWidth = 300
        label.numberOfLines = 0
        
        return label
    }
    
    private func createTextView(font: UIFont, message: String) -> UITextView {
        let textView = UITextView()
        textView.backgroundColor = .clear
        textView.font = font
        textView.text = message
        textView.textColor = .red
        
        textView.translatesAutoresizingMaskIntoConstraints = false
        
        textView.layer.borderWidth = 1
        textView.layer.borderColor = UIColor.red.cgColor
        
        textView.contentInset = .zero
        textView.contentInsetAdjustmentBehavior = .never
        
        textView.textContainerInset = .zero
        textView.textContainer.lineFragmentPadding = 0
        textView.sizeToFit()
        textView.layoutManager.usesFontLeading = false

        return textView
    }
}

PlaygroundPage.current.liveView = MyViewController()
